@@ -58,7 +58,7 @@
  * For testing this code, the following commands should work:
  *
  * exo-open --launch WebBrowser https://xfce.org (bug #5461).
- * exo-open --launch WebBrowser https://xfce.org bugzilla.xfce.org 'http://www.google.com/search?q=what is a space' 'https://wiki.xfce.org'
+ * exo-open --launch WebBrowser https://xfce.org gitlab.xfce.org 'http://www.google.com/search?q=what is a space' 'https://wiki.xfce.org'
  * exo-open https://xfce.org
  * exo-open --launch TerminalEmulator ./script.sh 'something with a space' 'nospace' (bug #5132).
  * exo-open --launch TerminalEmulator ssh -l username some.host.com
@@ -156,6 +156,16 @@ exo_open_launch_desktop_file (const gchar *arg)
   if (G_UNLIKELY (gfile == NULL))
     return FALSE;
 
+  /* Only execute local .desktop files to prevent execution of malicious launchers from  foreign locations */
+  if (g_file_has_uri_scheme (gfile, "file") == FALSE)
+    {
+      char *uri = g_file_get_uri (gfile);
+      g_warning ("Execution of remote .desktop file '%s' was skipped due to security concerns.", uri);
+      g_object_unref (gfile);
+      g_free (uri);
+      return FALSE;
+    }
+
   /* load the contents of the file */
   result = g_file_load_contents (gfile, NULL, &contents, &length, NULL, NULL);
   g_object_unref (G_OBJECT (gfile));
@@ -457,6 +467,8 @@ main (gint argc, gchar **argv)
 {
   GOptionContext  *context;
   GtkWidget       *dialog;
+  GtkWidget       *message_area;
+  GtkWidget       *label;
   GError          *err = NULL;
   gchar           *parameter, *quoted;
   gint             result = EXIT_SUCCESS;
@@ -495,24 +507,7 @@ main (gint argc, gchar **argv)
   gtk_window_set_default_icon_name ("preferences-desktop-default-applications");
 
   /* check what to do */
-  if (G_UNLIKELY (opt_help))
-    {
-      usage ();
-    }
-  else if (G_UNLIKELY (opt_version))
-    {
-      g_print ("%s %s\n\n", g_get_prgname (), PACKAGE_VERSION);
-      g_print (_("Copyright (c) %s\n"
-                 "        os-cillation e.K. All rights reserved.\n\n"
-                 "Written by Benedikt Meurer <benny@xfce.org>.\n\n"),
-               "2005-2007");
-      g_print (_("%s comes with ABSOLUTELY NO WARRANTY,\n"
-                 "You may redistribute copies of %s under the terms of\n"
-                 "the GNU Lesser General Public License which can be found in the\n"
-                 "%s source package.\n\n"), g_get_prgname (), g_get_prgname (), PACKAGE_TARNAME);
-      g_print (_("Please report bugs to <%s>.\n"), PACKAGE_BUGREPORT);
-    }
-  else if (G_LIKELY (opt_launch != NULL))
+  if (G_LIKELY (opt_launch != NULL))
     {
       if (argc > 1)
         {
@@ -597,11 +592,21 @@ main (gint argc, gchar **argv)
                 {
                   /* display an error dialog */
                   dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                                   _("Failed to open URI \"%s\"."), uri);
+                                                   _("Failed to open URI."));
                   if (startup_id != NULL)
                     gtk_window_set_startup_id (GTK_WINDOW (dialog), startup_id);
                   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s.", err->message);
                   g_error_free (err);
+
+                  /* add full URI to message area */
+                  message_area = gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog));
+                  label = gtk_label_new (uri);
+                  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+                  gtk_label_set_line_wrap_mode (GTK_LABEL (label), PANGO_WRAP_CHAR);
+                  gtk_label_set_max_width_chars (GTK_LABEL (label), 50);
+                  gtk_container_add (GTK_CONTAINER (message_area), label);
+                  gtk_widget_show (label);
+
                   gtk_dialog_run (GTK_DIALOG (dialog));
                   gtk_widget_destroy (dialog);
                 }
@@ -612,6 +617,23 @@ main (gint argc, gchar **argv)
           g_free (uri);
         }
     }
+  else if (G_UNLIKELY (opt_help))
+    {
+      usage ();
+    }
+  else if (G_UNLIKELY (opt_version))
+    {
+      g_print ("%s %s\n\n", g_get_prgname (), PACKAGE_VERSION);
+      g_print (_("Copyright (c) %s\n"
+                 "        os-cillation e.K. All rights reserved.\n\n"
+                 "Written by Benedikt Meurer <benny@xfce.org>.\n\n"),
+               "2005-2007");
+      g_print (_("%s comes with ABSOLUTELY NO WARRANTY,\n"
+                 "You may redistribute copies of %s under the terms of\n"
+                 "the GNU Lesser General Public License which can be found in the\n"
+                 "%s source package.\n\n"), g_get_prgname (), g_get_prgname (), PACKAGE_TARNAME);
+      g_print (_("Please report bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+    }
   else
     {
       result = EXIT_FAILURE;
